from insights.parsers.systemd import config

SystemdDocker = config.SystemdDocker
SystemdLogindConf = config.SystemdLogindConf
SystemdOpenshiftNode = config.SystemdOpenshiftNode
SystemdSystemConf = config.SystemdSystemConf
